@@ -19,16 +19,26 @@
  */
 package org.sejda.core.service;
 
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 import static org.sejda.TestUtils.encryptedAtRest;
 
 import java.io.IOException;
 
 import org.junit.Ignore;
 import org.junit.Test;
+import org.sejda.model.optimization.OptimizationPolicy;
 import org.sejda.model.output.ExistingOutputPolicy;
 import org.sejda.model.parameter.SimpleSplitParameters;
 import org.sejda.model.pdf.PdfVersion;
 import org.sejda.model.pdf.page.PredefinedSetOfPages;
+import org.sejda.sambox.cos.COSName;
+import org.sejda.sambox.pdmodel.PDResources;
+import org.sejda.sambox.pdmodel.graphics.state.PDExtendedGraphicsState;
+import org.sejda.sambox.pdmodel.graphics.state.PDSoftMask;
 
 /**
  * @author Andrea Vacondio
@@ -109,11 +119,46 @@ public abstract class SimpleSplitTaskTest extends BaseTaskTest<SimpleSplitParame
         testContext.assertTaskCompleted();
         testContext.assertOutputSize(19);
         testContext.assertOutputContainsFilenames("1_short-test-file.pdf", "3_short-test-file.pdf",
-                "1_medium-test-file.pdf", "3_medium-test-file.pdf", "5_medium-test-file.pdf", "7_medium-test-file.pdf", "9_medium-test-file.pdf",
-                "11_medium-test-file.pdf", "13_medium-test-file.pdf", "15_medium-test-file.pdf", "17_medium-test-file.pdf", "19_medium-test-file.pdf",
-                "21_medium-test-file.pdf", "23_medium-test-file.pdf", "25_medium-test-file.pdf", "27_medium-test-file.pdf", "29_medium-test-file.pdf",
-                "31_medium-test-file.pdf", "33_medium-test-file.pdf"
-        );
+                "1_medium-test-file.pdf", "3_medium-test-file.pdf", "5_medium-test-file.pdf", "7_medium-test-file.pdf",
+                "9_medium-test-file.pdf", "11_medium-test-file.pdf", "13_medium-test-file.pdf",
+                "15_medium-test-file.pdf", "17_medium-test-file.pdf", "19_medium-test-file.pdf",
+                "21_medium-test-file.pdf", "23_medium-test-file.pdf", "25_medium-test-file.pdf",
+                "27_medium-test-file.pdf", "29_medium-test-file.pdf", "31_medium-test-file.pdf",
+                "33_medium-test-file.pdf");
+    }
+
+    @Test
+    // make sure we don't loose the mask in case of shared pages resources and shared mask/forms resources, optimization triggered.
+    public void noDataLoss() throws IOException {
+        parameters = new SimpleSplitParameters(PredefinedSetOfPages.ALL_PAGES);
+        parameters.setCompress(true);
+        parameters.setVersion(PdfVersion.VERSION_1_6);
+        parameters.addSource(customInput("pdf/shared_res_form_extgs_softmask.pdf"));
+        parameters.setExistingOutputPolicy(ExistingOutputPolicy.OVERWRITE);
+        parameters.setOutputPrefix("[FILENUMBER]-chuck");
+        parameters.setOptimizationPolicy(OptimizationPolicy.AUTO);
+        testContext.directoryOutputTo(parameters);
+        execute(parameters);
+        testContext.assertTaskCompleted();
+        testContext.assertOutputSize(2).forPdfOutput("1-chuck.pdf", d -> {
+            try {
+                PDResources pageRes = d.getPage(0).getResources();
+                assertTrue(pageRes.isImageXObject(COSName.getPDFName("Im1")));
+                assertTrue(pageRes.isFormXObject(COSName.getPDFName("Form2")));
+                assertFalse(pageRes.isImageXObject(COSName.getPDFName("Im3")));
+                assertNull(pageRes.getExtGState(COSName.getPDFName("gs2")));
+                PDExtendedGraphicsState gs1 = pageRes.getExtGState(COSName.getPDFName("gs1"));
+                assertNotNull(gs1);
+                PDSoftMask softMask = gs1.getSoftMask();
+                assertNotNull(softMask);
+                PDResources maskRes = softMask.getGroup().getResources();
+                assertTrue(maskRes.isImageXObject(COSName.getPDFName("Im1")));
+                assertTrue(maskRes.isImageXObject(COSName.getPDFName("Im2")));
+                assertFalse(maskRes.isImageXObject(COSName.getPDFName("Im3")));
+            } catch (IOException e) {
+                fail();
+            }
+        });
     }
 
     @Test
