@@ -22,12 +22,11 @@ package org.sejda.model.parameter;
 
 import org.apache.commons.lang3.builder.EqualsBuilder;
 import org.apache.commons.lang3.builder.HashCodeBuilder;
-import org.sejda.model.parameter.base.SinglePdfSourceSingleOutputParameters;
+import org.sejda.commons.collection.NullSafeSet;
+import org.sejda.model.parameter.base.MultiplePdfSourceMultipleOutputParameters;
 import org.sejda.model.validation.constraint.NotEmpty;
-import org.sejda.model.validation.constraint.SingleOutputAllowedExtensions;
 
-import java.util.HashMap;
-import java.util.Map;
+import java.util.*;
 
 /**
  * Parameter class for the set metadata manipulation.
@@ -35,11 +34,12 @@ import java.util.Map;
  * @author Andrea Vacondio
  * 
  */
-@SingleOutputAllowedExtensions
-public final class SetMetadataParameters extends SinglePdfSourceSingleOutputParameters {
+public final class SetMetadataParameters extends MultiplePdfSourceMultipleOutputParameters {
 
     @NotEmpty
     private final Map<String, String> metadata = new HashMap<String, String>();
+    private final Set<String> toRemove = new NullSafeSet<>();
+    private Boolean updateCreatorProducerModifiedDate = true;
 
     public Map<String, String> getMetadata() {
         return metadata;
@@ -52,10 +52,31 @@ public final class SetMetadataParameters extends SinglePdfSourceSingleOutputPara
     public void put(String key, String metadata) {
         this.metadata.put(key, metadata);
     }
+    
+    public void addFieldToRemove(String fieldName) {
+        toRemove.add(fieldName);
+    }
+
+    public void addFieldsToRemove(Collection<String> fieldNames) {
+        toRemove.addAll(fieldNames);
+    }
+
+    public Set<String> getToRemove() {
+        return toRemove;
+    }
+
+    public Boolean isUpdateProducerModifiedDate() {
+        return updateCreatorProducerModifiedDate;
+    }
+
+    public void setUpdateCreatorProducerModifiedDate(Boolean updateCreatorProducerModifiedDate) {
+        this.updateCreatorProducerModifiedDate = updateCreatorProducerModifiedDate;
+    }
 
     @Override
     public int hashCode() {
-        return new HashCodeBuilder().appendSuper(super.hashCode()).append(metadata).toHashCode();
+        return new HashCodeBuilder().appendSuper(super.hashCode()).append(metadata).append(toRemove)
+                .append(updateCreatorProducerModifiedDate).toHashCode();
     }
 
     @Override
@@ -68,6 +89,8 @@ public final class SetMetadataParameters extends SinglePdfSourceSingleOutputPara
         }
         SetMetadataParameters parameter = (SetMetadataParameters) other;
         return new EqualsBuilder().appendSuper(super.equals(other)).append(metadata, parameter.metadata)
+                .append(toRemove, parameter.toRemove)
+                .append(updateCreatorProducerModifiedDate, parameter.updateCreatorProducerModifiedDate)
                 .isEquals();
     }
 }
