@@ -18,7 +18,8 @@
  */
 package org.sejda.impl.sambox;
 
-import static org.sejda.common.ComponentsUtility.nullSafeCloseQuietly;
+import static java.util.Optional.ofNullable;
+import static org.sejda.commons.util.IOUtils.closeQuietly;
 import static org.sejda.core.notification.dsl.ApplicationEventsNotifier.notifyEvent;
 import static org.sejda.core.support.io.IOUtils.createTemporaryBuffer;
 import static org.sejda.core.support.io.model.FileOutput.file;
@@ -69,13 +70,11 @@ public class RotateTask extends BaseTask<RotateParameters> {
 
     @Override
     public void execute(RotateParameters parameters) throws TaskException {
-        int currentStep = 0;
 
         for (int sourceIndex = 0; sourceIndex < parameters.getSourceList().size(); sourceIndex++) {
             PdfSource<?> source = parameters.getSourceList().get(sourceIndex);
 
-            executionContext().assertTaskNotCancelled();
-            currentStep++;
+            int fileNumber = executionContext().incrementAndGetOutputDocumentsCounter();
             LOG.debug("Opening {}", source);
             try {
                 documentHandler = source.open(documentLoader);
@@ -87,10 +86,9 @@ public class RotateTask extends BaseTask<RotateParameters> {
 
                 PdfRotator rotator = new PdfRotator(documentHandler.getUnderlyingPDDocument());
                 for (int page = 1; page <= documentHandler.getNumberOfPages(); page++) {
-                    executionContext().assertTaskNotCancelled();
                     Rotation rotation = parameters.getRotation(sourceIndex, page);
 
-                    if(rotation != Rotation.DEGREES_0) {
+                    if (rotation != Rotation.DEGREES_0) {
                         try {
                             rotator.rotate(page, rotation);
                         } catch (PageNotFoundException e) {
@@ -104,16 +102,19 @@ public class RotateTask extends BaseTask<RotateParameters> {
 
                 documentHandler.setVersionOnPDDocument(parameters.getVersion());
                 documentHandler.setCompress(parameters.isCompress());
-                documentHandler.savePDDocument(tmpFile);
+                documentHandler.savePDDocument(tmpFile, parameters.getOutput().getEncryptionAtRestPolicy());
+
+                String outName = ofNullable(parameters.getSpecificResultFilename(fileNumber)).orElseGet(() -> {
+                    return nameGenerator(parameters.getOutputPrefix())
+                            .generate(nameRequest().originalName(source.getName()).fileNumber(fileNumber));
+                });
 
-                String outName = nameGenerator(parameters.getOutputPrefix())
-                        .generate(nameRequest().originalName(source.getName()).fileNumber(currentStep));
                 outputWriter.addOutput(file(tmpFile).name(outName));
             } finally {
-                nullSafeCloseQuietly(documentHandler);
+                closeQuietly(documentHandler);
             }
 
-            notifyEvent(executionContext().notifiableTaskMetadata()).stepsCompleted(currentStep).outOf(totalSteps);
+            notifyEvent(executionContext().notifiableTaskMetadata()).stepsCompleted(fileNumber).outOf(totalSteps);
         }
 
         parameters.getOutput().accept(outputWriter);
@@ -122,7 +123,7 @@ public class RotateTask extends BaseTask<RotateParameters> {
 
     @Override
     public void after() {
-        nullSafeCloseQuietly(documentHandler);
+        closeQuietly(documentHandler);
     }
 
 }
\ No newline at end of file
