@@ -51,6 +51,7 @@ import org.sejda.sambox.pdmodel.PDPageContentStream;
 import org.sejda.sambox.pdmodel.graphics.PDXObject;
 import org.sejda.sambox.pdmodel.graphics.form.PDFormXObject;
 import org.sejda.sambox.pdmodel.graphics.image.PDImageXObject;
+import org.sejda.sambox.pdmodel.graphics.image.UnsupportedImageFormatException;
 import org.sejda.sambox.pdmodel.graphics.image.UnsupportedTiffImageException;
 import org.sejda.sambox.pdmodel.graphics.state.PDExtendedGraphicsState;
 import org.sejda.sambox.util.Matrix;
@@ -59,6 +60,8 @@ import org.sejda.sambox.util.filetypedetector.FileTypeDetector;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import net.coobird.thumbnailator.Thumbnails;
+
 public class PageImageWriter {
     private static final Logger LOG = LoggerFactory.getLogger(PageImageWriter.class);
 
@@ -92,8 +95,9 @@ public class PageImageWriter {
             PDPageContentStream.AppendMode mode, PDExtendedGraphicsState gs, boolean resetContext, int rotation)
             throws TaskIOException {
         try (PDPageContentStream contentStream = new PDPageContentStream(document, page, mode, true, resetContext)) {
-            AffineTransform at = new AffineTransform(width, 0, 0, height, (float) position.getX(), (float) position.getY());
-            if(rotation != 0) {
+            AffineTransform at = new AffineTransform(width, 0, 0, height, (float) position.getX(),
+                    (float) position.getY());
+            if (rotation != 0) {
                 at.rotate(Math.toRadians(rotation));
             }
 
@@ -111,19 +115,27 @@ public class PageImageWriter {
         try {
             return createFromSeekableSource(source.getSeekableSource(), source.getName());
         } catch (Exception e) {
-            throw new TaskIOException("An error occurred creating PDImageXObject from file source: " + source.getName(), e);
+            throw new TaskIOException("An error occurred creating PDImageXObject from file source: " + source.getName(),
+                    e);
         }
     }
 
-    public static PDImageXObject createFromSeekableSource(SeekableSource original, String name) throws TaskIOException, IOException {
+    public static PDImageXObject createFromSeekableSource(SeekableSource original, String name)
+            throws TaskIOException, IOException {
         SeekableSource source = original;
-        Optional<SeekableSource> maybeConvertedFile = convertCMYKJpegIf(source);
-        if(maybeConvertedFile.isPresent()) {
+
+        Optional<SeekableSource> maybeConvertedFile = convertExifRotatedIf(source, name);
+        if (maybeConvertedFile.isPresent()) {
             source = maybeConvertedFile.get();
         }
 
-        maybeConvertedFile = convertICCGrayPngIf(source);
-        if(maybeConvertedFile.isPresent()) {
+        maybeConvertedFile = convertCMYKJpegIf(source, name);
+        if (maybeConvertedFile.isPresent()) {
+            source = maybeConvertedFile.get();
+        }
+
+        maybeConvertedFile = convertICCGrayPngIf(source, name);
+        if (maybeConvertedFile.isPresent()) {
             source = maybeConvertedFile.get();
         }
 
@@ -160,7 +172,8 @@ public class PageImageWriter {
         }
     }
 
-    public static SeekableSource convertImageTo(SeekableSource source, String format) throws IOException, TaskIOException {
+    public static SeekableSource convertImageTo(SeekableSource source, String format)
+            throws IOException, TaskIOException {
         BufferedImage image = ImageIO.read(source.asNewInputStream());
         File tmpFile = IOUtils.createTemporaryBuffer("." + format);
         ImageOutputStream outputStream = new FileImageOutputStream(tmpFile);
@@ -182,10 +195,45 @@ public class PageImageWriter {
     }
 
     /**
-     * Checks if the input file is a JPEG using CMYK
-     * If that's the case, converts to RGB and returns the file path
+     * Checks if the input file has exit rotation If that's the case, converts to rotated image without exif rotation
      */
-    private static Optional<SeekableSource> convertCMYKJpegIf(SeekableSource source) throws IOException, TaskIOException {
+    private static Optional<SeekableSource> convertExifRotatedIf(SeekableSource source, String name)
+            throws IOException, TaskIOException {
+        int degrees = ExifHelper.getRotationBasedOnExifOrientation(source.asNewInputStream());
+        
+        if (degrees > 0) {
+            BufferedImage orig = ImageIO.read(source.asNewInputStream());
+            
+            if(orig == null) {
+                FileType type = FileTypeDetector.detectFileType(source);
+                throw new UnsupportedImageFormatException(type, name, null);
+            }
+            
+            BufferedImage result = Thumbnails.of(orig).scale(1).rotate(degrees).asBufferedImage();
+
+            File tmpFile = IOUtils.createTemporaryBufferWithName(name);
+            ImageIO.write(result, getImageIOSaveFormat(source), tmpFile);
+            return Optional.of(SeekableSources.seekableSourceFrom(tmpFile));
+
+        }
+        
+        return Optional.empty();
+    }
+
+    private static String getImageIOSaveFormat(SeekableSource source) {
+        FileType fileType = getFileType(source);
+        if (fileType == FileType.JPEG) {
+            return "jpg";
+        }
+
+        return "png";
+    }
+
+    /**
+     * Checks if the input file is a JPEG using CMYK If that's the case, converts to RGB and returns the file path
+     */
+    private static Optional<SeekableSource> convertCMYKJpegIf(SeekableSource source, String name)
+            throws IOException, TaskIOException {
         try {
             if (FileType.JPEG.equals(getFileType(source))) {
                 try (ImageInputStream iis = ImageIO.createImageInputStream(source.asNewInputStream())) {
@@ -194,7 +242,7 @@ public class PageImageWriter {
                     try {
                         ImageIO.setUseCache(false);
                         reader.setInput(iis);
-                        for (Iterator<ImageTypeSpecifier> it = reader.getImageTypes(0); it.hasNext(); ) {
+                        for (Iterator<ImageTypeSpecifier> it = reader.getImageTypes(0); it.hasNext();) {
                             ImageTypeSpecifier typeSpecifier = it.next();
                             if (typeSpecifier.getColorModel().getColorSpace().getType() == ColorSpace.TYPE_CMYK) {
                                 isCmyk = true;
@@ -207,7 +255,7 @@ public class PageImageWriter {
                             // twelvemonkeys JPEG plugin already converts it to rgb when reading the image
                             // just write it out
                             BufferedImage image = reader.read(0);
-                            File tmpFile = IOUtils.createTemporaryBuffer();
+                            File tmpFile = IOUtils.createTemporaryBufferWithName(name);
                             ImageIO.write(image, "jpg", tmpFile);
                             return Optional.of(SeekableSources.seekableSourceFrom(tmpFile));
                         }
@@ -217,7 +265,7 @@ public class PageImageWriter {
                 }
             }
         } catch (IIOException e) {
-            if(e.getMessage().startsWith("Not a JPEG stream")) {
+            if (e.getMessage().startsWith("Not a JPEG stream")) {
                 // this was a different image format with a JPEG extension
             } else {
                 throw e;
@@ -228,10 +276,10 @@ public class PageImageWriter {
     }
 
     /**
-     * Checks if the input file is a PNG using ICC Gray color model
-     * If that's the case, converts to RGB and returns the file path
+     * Checks if the input file is a PNG using ICC Gray color model If that's the case, converts to RGB and returns the file path
      */
-    private static Optional<SeekableSource> convertICCGrayPngIf(SeekableSource source) throws IOException, TaskIOException {
+    private static Optional<SeekableSource> convertICCGrayPngIf(SeekableSource source, String name)
+            throws IOException, TaskIOException {
         try {
             if (FileType.PNG.equals(getFileType(source))) {
                 try (ImageInputStream iis = ImageIO.createImageInputStream(source.asNewInputStream())) {
@@ -240,10 +288,11 @@ public class PageImageWriter {
                     try {
                         ImageIO.setUseCache(false);
                         reader.setInput(iis);
-                        for (Iterator<ImageTypeSpecifier> it = reader.getImageTypes(0); it.hasNext(); ) {
+                        for (Iterator<ImageTypeSpecifier> it = reader.getImageTypes(0); it.hasNext();) {
                             ImageTypeSpecifier typeSpecifier = it.next();
                             ColorSpace colorSpace = typeSpecifier.getColorModel().getColorSpace();
-                            if (colorSpace instanceof ICC_ColorSpace && ((ICC_ColorSpace) colorSpace).getProfile() instanceof ICC_ProfileGray) {
+                            if (colorSpace instanceof ICC_ColorSpace
+                                    && ((ICC_ColorSpace) colorSpace).getProfile() instanceof ICC_ProfileGray) {
                                 isICCGray = true;
                                 break;
                             }
@@ -254,7 +303,7 @@ public class PageImageWriter {
                             // convert to rgb
                             BufferedImage original = reader.read(0);
                             BufferedImage rgb = toARGB(original);
-                            File tmpFile = IOUtils.createTemporaryBuffer();
+                            File tmpFile = IOUtils.createTemporaryBufferWithName(name);
                             ImageIO.write(rgb, "png", tmpFile);
                             return Optional.of(SeekableSources.seekableSourceFrom(tmpFile));
                         }
